﻿using System;
using System.IO;
using NLog;
using NLog.Config;
using NLog.Layouts;
using NLog.Targets;

namespace pdfforge.PDFCreator.Shared.Helper.Logging
{
    internal class FileLogger : ILogger
    {
        private const string TraceLogLayout =
            "${shortdate} ${date:format=HH\\:mm\\:ss.ffff} [${level}] ${processid}-${threadid} (${threadname}) ${callsite}: ${message}";

        private const string ShortLogLayout =
            "${shortdate} ${date:format=HH\\:mm\\:ss.ffff} [${level}] ${callsite}: ${message}";

        private readonly FileTarget _fileTarget;
        private readonly string _logFile;
        private LoggingRule _loggingRule;

        public FileLogger(string applicationName, LogLevel logLevel, string logFilePath = null)
        {
            var config = new LoggingConfiguration();

            _fileTarget = new FileTarget();
            _fileTarget.Layout = ShortLogLayout;
            _fileTarget.ArchiveAboveSize = 10485760; // 10MB
            _fileTarget.MaxArchiveFiles = 2;

            _fileTarget.Layout = GetLayoutForLogLevel(logLevel);

            _logFile = GetLogFilePath(applicationName, logFilePath);
            _fileTarget.FileName = _logFile;

            config.AddTarget("file", _fileTarget);

            _loggingRule = new LoggingRule("*", logLevel, _fileTarget);
            config.LoggingRules.Add(_loggingRule);

            LogManager.Configuration = config;
        }

        public void ChangeLogLevel(LogLevel logLevel)
        {
            _fileTarget.Layout = GetLayoutForLogLevel(logLevel);

            LogManager.Configuration.LoggingRules.Remove(_loggingRule);

            _loggingRule = new LoggingRule("*", logLevel, _fileTarget);
            LogManager.Configuration.LoggingRules.Add(_loggingRule);

            LogManager.Configuration.Reload();
        }

        public string GetLogPath()
        {
            return _logFile;
        }

        private Layout GetLayoutForLogLevel(LogLevel logLevel)
        {
            return (logLevel == LogLevel.Trace) ? TraceLogLayout : ShortLogLayout;
        }

        private string GetLogFilePath(string applicationName, string logFilePath)
        {
            var path = logFilePath ?? Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), applicationName);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            var fileName = Path.Combine(path, applicationName + ".log");

            return fileName;
        }
    }
}